
/*
   Background Sound Player

   Copyright 1993 by Kurt Siegl <siegl@risc.uni-linz.ac.at>
   Permission to use, modify, copy and distribute this software without
   fee is hereby granted as long as this notice is left here.
*/

#include <stdio.h>
#include <malloc.h>
#include <unistd.h>
#include <stdlib.h>
#include <limits.h>
#include <fcntl.h>
#include <string.h>
#ifdef linux
#include <linux/soundcard.h>
#endif
#include <signal.h>
#include <termio.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include "sndconfig.h"

#define TRUE 1
#define FALSE 0

static int play_sound;
static int audio, abuf_size, play_fd, play_speed;
static char *audiobuf;

static void StopSoundI ();
static void StartSoundI ();

static struct shm_sound *shared_data;
static int returnpid;

static void play ();

static void 
terminate (x)
     int x;
{
  switch (x)
    {
    case (0):
      kill (returnpid, SIGUSR1);
      break;
    case (1):
      kill (returnpid, SIGUSR2);
    default:
      exit (x);
    }
}

int 
main (argc, argv)
     int argc;
     char **argv;
{
  int value;
  struct itimerval udt;
  int shmid;

  if (argc != 3)		/* Usage: bgsndplay shmID parrentPID */
    {
      fprintf (stderr, "This program requires 2 arguments.\n");
      fprintf (stderr, "But why are you running it anyway :->\n");
      terminate (-1);
    }

  shmid = atoi (argv[1]);
  returnpid = atoi (argv[2]);

  audio = open (AUDIO, O_WRONLY, 0);
  if (audio == -1)
    {
      perror (AUDIO);
      terminate (1);
    }
 
#ifdef linux
  abuf_size = 0;

  value = FRAG_SIZE; 
  ioctl(audio, SNDCTL_DSP_SETFRAGMENT, &value);

#if 0
  value = DEFAULT_SAMPLE_SIZE;
  if (ioctl (audio, SNDCTL_DSP_SAMPLESIZE, &value) == -1)
    {
      fprintf (stderr, "Unable to set the sample size\n");
      perror (AUDIO);
     terminate (1);
    }
#endif

  value = DEFAULT_DSP_SPEED;
  if (ioctl(audio, SNDCTL_DSP_SPEED, &value))
  {
      fprintf (stderr, "Unable to set audio speed\n");
      perror (AUDIO);
      terminate (1);
  };

  value = STEREO;
  if (ioctl (audio, SNDCTL_DSP_STEREO, &value) == -1)
    {
      fprintf (stderr, "Unable to set mono/stereo flag\n");
      perror (AUDIO);
/*      terminate (1); */
    }

  ioctl (audio, SNDCTL_DSP_GETBLKSIZE, &abuf_size);
  if (abuf_size == -1) {
      perror (AUDIO);
      terminate (1);
    }

#else
  abuf_size = DEFAULT_BLKSIZE;
#endif

  if ((audiobuf = malloc (abuf_size)) == NULL)
    {
      fprintf (stderr, "Unable to allocate input/output buffer\n");
      terminate (1);
    }

  /*
  if ((shmid=shmget(shmKey,sizeof(struct shm_sound),0))==-1)
	{
	  perror ("SHMGET");
	  terminate(1);
	}
*/

  if ((shared_data = (struct shm_sound *) shmat (shmid, 0, 0)) == (void *) -1)
    {
      perror ("SHMAT");
      terminate (1);
    }

  shared_data->play_sound = FALSE;

  signal (SIGUSR1, StopSoundI);
  signal (SIGUSR2, StartSoundI);

  terminate (0);

  while (1)
    {
      if (shared_data->play_sound) 
	play();
      else
	pause ();
    }

  fprintf (stderr, "Shouldn't happen\n");
  exit (-1);
}

static void 
StopSoundI ()
{
  if (shared_data->play_sound)
    {
      close (play_fd);
      shared_data->play_sound = FALSE;
    }
  signal (SIGUSR1, StopSoundI);
}

static void 
StartSoundI ()
{
  char *name;

  StopSoundI ();
  name = shared_data->name;
  if ((play_fd = open (name, O_RDONLY, 0)) == -1)
    {
      perror (name);
      return;
    }
  shared_data->play_sound = TRUE;
  /*  play(); */
  signal (SIGUSR2, StartSoundI);
  return;
}


static void 
play ()
{
  int l;
  struct timeval t;

#ifdef DEBUG
  printf("Start\n");
#endif

  if ((l = read (play_fd, audiobuf, abuf_size))
      > 0)
    {
      if (write (audio, audiobuf, l) != l)
	{
	  if (errno != EINTR ) {
	    perror (AUDIO);
	    return;
	  }
	}
      write (audio, (char *) 0, 0);
#ifdef linux
      if (l < abuf_size)
	{
	  ioctl (audio, SNDCTL_DSP_POST);
	}
#endif
    }
  else
    {
      if (l == -1)
	{
	  perror ("Input file");
	  return;
	}
      close (play_fd);
      shared_data->play_sound = FALSE;	/* Stop */
      return;
    }
#ifdef DEBUG
  printf("End\n");
#endif
}

